/*
*  Name:      base_type.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL data type
*  Date:      $Date: 2002/09/25 23:13:45 $
*  Revision:  $Revision: 1.2 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQLC_BASE_TYPE__
#define _UESQLC_BASE_TYPE__

#include <mpcl/string.hh>


///  Name-space for UESQL.
namespace uesql
{

  ///  Base SQL data type. It is used as base class for embedded variables.
  class TBaseType
  {

    private:

      ///  If true, then data retrieved from SQL server, is null.
      bool   gIsNull;

      ///  Buffer for getting values passively.
      mutable char*   pcBuffer;


    public:

      ///  Size type.
      typedef
        unsigned int
        size_type;


    protected:

      //
      //  C O N S T R U C T O R S
      //

      /**
      *  Sets null state of the instance.
      *  @param gIS_NULL Null state.
      */
      void setNull (bool gIS_NULL)
      {
        gIsNull = gIS_NULL;
      }


    public:
      
      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TBaseType (void)  :
        gIsNull  (true) ,
        pcBuffer (NULL) {}

      ///  Destroys the instance.
      virtual ~TBaseType (void)
      {
        if ( pcBuffer )
        {
          delete [] pcBuffer;
        }
      }
      
      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      void updateFromBuffer (void)
      {
        if ( pcBuffer )
        {
          set ((const char*) pcBuffer);
        }
      }

      /**
      *  Assigns value \a pkcVALUE.
      *  @param pkcVALUE Source value.
      */
      virtual TBaseType& set (const char* pkcVALUE) = 0;
      
      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      virtual TBaseType& set (const std::string& rkyVALUE) = 0;


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string with the value of the instance. If the instance holds
      *  a null value, then the string 'NULL' will be printed.
      *  @return The result of converting this instance to string.
      */
      virtual mpcl::TString get (void) const = 0;

      /**
      *  Returns a buffer where you can assign a value of, at most,
      *  \a kzSIZE octets.
      *  @param kzSIZE Octet count.
      *  @return A pointer to the buffer.
      */
      void* getBuffer (const size_type kzSIZE) const
      {
        if ( pcBuffer )
        {
          delete [] pcBuffer;
        }
        pcBuffer = new char [kzSIZE + 1];
        return pcBuffer;
      }

      /**
      *  Returns true if the last data retrieved has been a NULL value.
      *  @return True if the last data retrieved has been a NULL value.
      */
      bool isNull (void) const
      {
        return gIsNull;
      }

  };  // class TBaseType
  
}  // namespace uesql


#endif  // not _UESQLC_BASE_TYPE__
