/*
*  Name:      uesql_scanner.ll
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQL scanner 
*  Date:      $Date: 2002/09/16 20:09:35 $
*  Revision:  $Revision: 1.16 $
*
*  Comments:  The introducer instruction  ("extern")  for the embedded program, 
*             can not contain any C/C++ comment,  between  the first clause and
*             the opening brace.
*
*             Subsection 5.3:  <literal>.
*             Terminal  <nonquote character>  has  been  changed to include all
*             characters minus <quote> and <newline>.
*
*             Subsection 19.1. <embedded SQL host program>.
*             Terminal <embedded n-adic function call> has  been  introduced to
*             support, not only embedded variables,  but, n-adic function calls
*             (including name-spaces support).
*
*             Subsection 19.1. <embedded SQL host program>.
*             Terminal <embedded array>  has been introduced to support, multi-
*             dimensioned  arrays  (including name-spaces support).  The  array
*             indexes must be variables or constants.
*
*             Subsection 19.1. <embedded SQL host program>.
*             Terminal <embedded variable name>  has  been extended  to support
*             the access to struct/class member variables.
*
*             Subsection 19.4. <embedded SQL C program>.
*             Terminal <C host identifier> has been modified to follow the ANSI
*             C standard.
*
*  Copyright (C) 1996-2000  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*  Copyright (C) 1999-2000  Dedalo Ingenieros, S.L.
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*/


/*
*  L E X   D E F I N I T I O N S
*/

%{

#include "application.hh"
#include "defs.hh"
#include "stack_value.hh"
#include "uesql_parser.h"


//
//  E X P O R T E D   L E X   M A C R O S
//

//
//  Makes LEX line buffer lenght ('yytext' length).
//
#ifdef YYLMAX
#undef YYLMAX
#endif
#define YYLMAX (1048576L)

//
//  Makes FLEX not interactive (so it doesn't define 'isatty()').
//  
#define YY_NEVER_INTERACTIVE (1)

//
//  E X P O R T E D   V A R I A B L E S
//

extern TApplication*   ptApplication;


//
//  E X P O R T E D   M A C R O S
//

#define ECHO_CACHING  { ptApplication->outputCode (yytext, yyleng); }

#define DEBUG_TERMINAL(pccTERMINAL)           \
{                                             \
  ptApplication->pushTokenName (pccTERMINAL); \
  ptApplication->printDebugInformation();     \
}

#define RETURN_TERMINAL(iTERMINAL) \
{                                  \
  DEBUG_TERMINAL (#iTERMINAL);     \
  return iTERMINAL;                \
}

#define RETURN_LITERAL_TERMINAL(iTERMINAL, pkcTEXT) \
{                                                   \
  yylval.yCode = pkcTEXT;                           \
  DEBUG_TERMINAL (#iTERMINAL);                      \
  return iTERMINAL;                                 \
}

#define RETURN_STRING_LITERAL_TERMINAL(iTERMINAL, pkcTEXT)       \
{                                                                \
  yylval.yCode = ptApplication->collapseStringLiteral (pkcTEXT); \
  DEBUG_TERMINAL (#iTERMINAL);                                   \
  return iTERMINAL;                                              \
}

%}


/*
*  Definitions for embedded introducer and
*  C language comments removing.
*/

one_line_C_comment		("//".*\n)
multi_line_C_comment_begin	("/*")
multi_line_C_comment_end	("*/")
C_separator			(({one_line_C_comment}|{C_spaces})*)
C_spaces		       	([[:space:]]+)
extern_identifier		("UESQL::")
server_class		       	([[:alpha:]][[:alnum:][:punct:]]*)
embedded_introducer		([[:blank:]]*"extern"{C_separator}\"{extern_identifier}{server_class}\"{C_separator}"{")


/*                             
*  Section 5. Lexical Elements.
*/

/*                                            
*  Subsection 5.1. <SQL terminal character>. 
*/

SQL_terminal_character		({SQL_language_character}|{SQL_embedded_language_character})
SQL_embedded_language_character ({left_bracket}|{right_bracket})
SQL_language_character 		({simple_Latin_letter}|{digit}|{SQL_special_character})
simple_Latin_letter		({simple_Latin_upper_case_letter}|{simple_Latin_lower_case_letter})
simple_Latin_upper_case_letter	([A-Z])
simple_Latin_lower_case_letter	([a-z])
digit				([0-9])
/*
*  {SQL_delimited_special_character}
*
*  Warning:  This terminal isn't part of the specification
*            but it simplifies the definition of other 
*            terminals.
*/
SQL_delimited_special_character ({space}|{percent}|{ampersand}|{quote}|{left_paren}|{right_paren}|{asterisk}|{plus_sign}|{comma}|{minus_sign}|{period}|{solidus}|{colon}|{semicolon}|{less_than_operator}|{equals_operator}|{greater_than_operator}|{question_mark}|{underscore}|{vertical_bar})
SQL_special_character		({double_quote}|{SQL_delimited_special_character})
space                   	" "
double_quote  			\"
percent				\%
ampersand			\&
quote				\'
left_paren			\(
right_paren			\)
asterisk			\*
plus_sign			\+
comma				\,
minus_sign			\-
period 				\.
solidus				\/
colon				\:
semicolon			\;
less_than_operator		\<
equals_operator			\=
greater_than_operator		\>
question_mark			\?
left_bracket			\[
right_bracket			\]
underscore			\_
vertical_bar			\|

/*
*  Auxiliar terminals:  second_level_identifier
*                       third_level_identifier
*                       fourth_level_identifier
*                       second_level_asterisk
*                       third_level_asterisk
*                       fourth_level_asterisk
*                       C_array_variable_index
*			C_array_constant_index
*			C_array_index
*                       C_variable
*                       C_string_literal
*                       C_numeric_literal
*/
second_level_identifier  ({identifier_body}{period}{identifier_body})
third_level_identifier   ({identifier_body}{period}{second_level_identifier})
fourth_level_identifier  ({identifier_body}{period}{third_level_identifier})
second_level_asterisk    ({identifier_body}{period}{asterisk})
third_level_asterisk     ({identifier_body}{period}{second_level_asterisk})
fourth_level_asterisk    ({identifier_body}{period}{third_level_asterisk})
C_array_variable_index   ({left_bracket}{C_spaces}*{C_host_identifier}{C_spaces}*{right_bracket})
C_array_constant_index   ({left_bracket}{C_spaces}*{unsigned_integer}{C_spaces}*{right_bracket})
C_array_index            ({C_array_variable_index}|{C_array_constant_index})
C_variable               (({ampersand}|{asterisk})*{C_spaces}*{C_host_identifier})
C_string_literal         ({double_quote}[^\"]*{double_quote})
C_numeric_literal        ({real_exact_numeric_literal}|{signed_integer}|{unsigned_integer})

/*                           
*  Subsection 5.2. <token> and <separator>.
*/

/*
*  Warning:  A new terminal ({tabulator}) has been added to {separator}
*            to let exist tabs in {separator}.
*/
identifier_body			({identifier_start}({underscore}|{identifier_part})*)
identifier_start		({simple_Latin_letter})
identifier_part			({identifier_start}|{digit})
delimited_identifier		({double_quote}{delimited_identifier_body}{double_quote})
delimited_identifier_body	({delimited_identifier_part}+)
delimited_identifier_part	({nondoublequote_character}|{doublequote_symbol})
nondoublequote_character	({simple_Latin_letter}|{digit}|{SQL_delimited_special_character})
doublequote_symbol		({double_quote}{double_quote})
not_equals_operator		("<>")
greater_than_or_equals_operator (">=")
less_than_or_equals_operator	("<=")
concatenation_operator		("||")
double_period			("..")
separator			(({comment}|{space}|{newline}|{tabulator})+)
comment				({comment_introducer}{comment_character}*{newline})
comment_character		({nonquote_character}|{quote})
comment_introducer		({minus_sign}{minus_sign}+)
newline				(\n)
tabulator			(\t)
not_like			((("N"|"n")("O"|"o")("T"|"t")){separator}(("L"|"l")("I"|"i")("K"|"k")("E"|"e")))


/*                           
*  Subsection 5.3. <literal>.
*/

/*
*  Warning:  Character set specification has been removed from these
*            terminals:
*
*            {character_string_literal}
*            {national_character_string_literal}
*            {bit_string_literal}
*            {hex_string_literal}
*/
character_string_literal		({quote}{character_representation}*{quote}({separator}+{quote}{character_representation}*{quote})*)
character_representation		({nonquote_character}|{quote_symbol})
nonquote_character			([^\'\n])
quote_symbol				({quote}{quote})
national_character_string_literal	(N{quote}{character_representation}*{quote}({separator}+{quote}{character_representation}*{quote})*)
bit_string_literal			(B{quote}{bit}*{quote}({separator}+{quote}{bit}*{quote})*)
hex_string_literal			(X{quote}{hexit}*{quote}({separator}+{quote}{hexit}*{quote})*)
bit					(0|1)
hexit					({digit}|A|B|C|D|E|F|a|b|c|d|e|f)
sign					({plus_sign}|{minus_sign})
approximate_numeric_literal		({mantissa}E{exponent})
mantissa				({real_exact_numeric_literal}|{unsigned_integer})
exponent				({signed_integer})
signed_integer				(({sign}{unsigned_integer})|({unsigned_integer}))
unsigned_integer			({digit}+)
date_string				({quote}{date_value}{quote})
time_string				({quote}{time_value}({time_zone_interval})?{quote})
timestamp_string			({quote}{date_value}{space}{time_value}({time_zone_interval})?{quote})
time_zone_interval			({sign}{hours_value}{colon}{minutes_value})
date_value				({years_value}{minus_sign}{months_value}{minus_sign}{days_value})
time_value				({hours_value}{colon}{minutes_value}{colon}{seconds_value})
interval_string				(({quote}{year_month_literal}{quote})|({quote}{day_time_literal}{quote}))
year_month_literal			(({years_value}({minus_sign}{months_value})?)|({months_value}))
day_time_literal			({day_time_interval}|{time_interval})
day_time_interval			({days_value}({space}{hours_value}({colon}{minutes_value}({colon}{seconds_value})?)?)?)
/*
*  {time_interval}
*
*  Warning:  This terminal has been modified to
*            simplify its definition.
*/
time_interval				((({hours_value}{colon})?{minutes_value}{colon})?{seconds_value})
years_value				({datetime_value})
months_value				({datetime_value})
days_value				({datetime_value})
hours_value				({datetime_value})
minutes_value				({datetime_value})
seconds_value				({seconds_integer_value}({period}({seconds_fraction})?)?)
seconds_integer_value			({unsigned_integer})
seconds_fraction			({unsigned_integer})
datetime_value				({unsigned_integer})

/*
*  Auxiliar terminals:  {real_exact_numeric_literal}
*/
real_exact_numeric_literal		(({unsigned_integer}{period}{unsigned_integer})|({unsigned_integer}{period})|({period}{unsigned_integer}))


/*
*  Subsection 19.1. <embedded SQL host program>.
*/

/*
*  Warning:  A new terminal ({multi_host_identifier}) has been added to let the
*            newly  added terminal {exception_target} to contain exceptions  to
*            have name-space qualified.
*
*  Warning:  It  has been added two terminals  ({embedded_n_adic_function_call}
*            and {function_parameter}) to parse embedded n-adic function calls.
*
*  Warning:  It has been added two terminals for parsing strings literals.
*/
name_space_separator                    ({colon}{colon})
multi_host_identifier                   ({C_host_identifier}({name_space_separator}{C_host_identifier})+)
host_identifier				({C_host_identifier})
function_parameter                      ({C_spaces}*({C_variable}|{C_string_literal}|{C_numeric_literal}){C_spaces}*)
embedded_n_adic_function_call           ({colon}{host_identifier}({name_space_separator}{host_identifier})*{C_spaces}*{left_paren}({function_parameter}({comma}{function_parameter})*)?{right_paren})
embedded_array                          ({colon}{host_identifier}({name_space_separator}{host_identifier})*{C_spaces}*{C_array_index}+)
embedded_variable_name			({colon}({ampersand}|{asterisk})*{host_identifier}({name_space_separator}{host_identifier})*({period}{host_identifier})*)


/*
*  Subsection 19.4. <embedded SQL C program>.
*/

C_host_identifier			(({simple_Latin_letter}|{underscore})({simple_Latin_letter}|{digit}|{underscore})*)


/*
*  Start Conditions.
*/
%start MULTILINE_C_COMMENT SC_SQL

%%

	/*
	*  L E X   R U L E S
	*/

<INITIAL>{embedded_introducer}		BEGIN (SC_SQL); ptApplication->handleTarget (yytext); RETURN_TERMINAL (EMBEDDED_SQL_BEGIN);
<INITIAL>{one_line_C_comment}		ECHO_CACHING;
<INITIAL>{multi_line_C_comment_begin}	ECHO_CACHING; BEGIN (MULTILINE_C_COMMENT); 
<INITIAL>\n				ECHO_CACHING;
<INITIAL>.				ECHO_CACHING;

<MULTILINE_C_COMMENT>[^\*]*                       ECHO_CACHING;
<MULTILINE_C_COMMENT>{multi_line_C_comment_end}   ECHO_CACHING; BEGIN (INITIAL);
<MULTILINE_C_COMMENT>"*"                          ECHO_CACHING;


	/*                                           
	*  Subsection 19.4: <embedded SQL C program>.
	*/

<SC_SQL>{embedded_n_adic_function_call}   RETURN_LITERAL_TERMINAL (EMBEDDED_N_ADIC_FUNCTION_CALL, yytext + 1);
<SC_SQL>{embedded_array}                  RETURN_LITERAL_TERMINAL (EMBEDDED_ARRAY,                yytext + 1);
<SC_SQL>{embedded_variable_name}          RETURN_LITERAL_TERMINAL (EMBEDDED_VARIABLE_NAME,        yytext + 1);

	/*
	*  Subsection 19.1. <embedded SQL host program>.
	*/
<SC_SQL>{multi_host_identifier}           RETURN_LITERAL_TERMINAL (MULTI_HOST_IDENTIFIER, yytext);

	/*
	*  Subsection 5.1: <SQL terminal character>. 
	*/

<SC_SQL>{left_paren}			RETURN_TERMINAL (LEFT_PAREN);
<SC_SQL>{right_paren}			RETURN_TERMINAL (RIGHT_PAREN);
<SC_SQL>{asterisk}			RETURN_TERMINAL (ASTERISK);
<SC_SQL>{plus_sign}			RETURN_TERMINAL (PLUS_SIGN);
<SC_SQL>{comma}				RETURN_TERMINAL (COMMA);
<SC_SQL>{minus_sign}			RETURN_TERMINAL (MINUS_SIGN);
<SC_SQL>{solidus}			RETURN_TERMINAL (SOLIDUS);
<SC_SQL>{semicolon}               	RETURN_TERMINAL (SEMICOLON);
<SC_SQL>{less_than_operator}		RETURN_TERMINAL (LESS_THAN_OPERATOR);
<SC_SQL>{equals_operator}		RETURN_TERMINAL (EQUALS_OPERATOR);
<SC_SQL>{greater_than_operator}		RETURN_TERMINAL (GREATER_THAN_OPERATOR);
<SC_SQL>{question_mark}			RETURN_TERMINAL (QUESTION_MARK);

	/*
	*  Subsection 5.2: <token> and <separator>. 
	*/
	
	/*
	*  Non-Terminal:  <reserved word>
	*
	*  RW_NOT_LIKE
	*
	*  Warning: This terminal isn't part of the specification.
	*/
<SC_SQL>{not_like}			RETURN_TERMINAL (RW_NOT_LIKE);

	/*
	*  Non-Terminal:  <non-reserved word>
	*/
<SC_SQL>"CATALOG_NAME"			RETURN_LITERAL_TERMINAL (NRW_CATALOG_NAME, yytext);
<SC_SQL>"CHARACTER_SET_CATALOG"		RETURN_LITERAL_TERMINAL (NRW_CHARACTER_SET_CATALOG, yytext);
<SC_SQL>"CHARACTER_SET_NAME"		RETURN_LITERAL_TERMINAL (NRW_CHARACTER_SET_NAME, yytext);
<SC_SQL>"CHARACTER_SET_SCHEMA" 		RETURN_LITERAL_TERMINAL (NRW_CHARACTER_SET_SCHEMA, yytext);
<SC_SQL>"CLASS_ORIGIN"			RETURN_LITERAL_TERMINAL (NRW_CLASS_ORIGIN, yytext);
<SC_SQL>"COLLATION_CATALOG"		RETURN_LITERAL_TERMINAL (NRW_COLLATION_CATALOG, yytext);
<SC_SQL>"COLLATION_NAME"		RETURN_LITERAL_TERMINAL (NRW_COLLATION_NAME, yytext);
<SC_SQL>"COLLATION_SCHEMA" 		RETURN_LITERAL_TERMINAL (NRW_COLLATION_SCHEMA, yytext);
<SC_SQL>"COLUMN_NAME"			RETURN_LITERAL_TERMINAL (NRW_COLUMN_NAME, yytext);
<SC_SQL>"COMMAND_FUNCTION" 		RETURN_LITERAL_TERMINAL (NRW_COMMAND_FUNCTION, yytext);
<SC_SQL>"COMMITTED"			RETURN_LITERAL_TERMINAL (NRW_COMMITTED, yytext);
<SC_SQL>"CONDITION_NUMBER" 		RETURN_LITERAL_TERMINAL (NRW_CONDITION_NUMBER, yytext);
<SC_SQL>"CONNECTION_NAME"		RETURN_LITERAL_TERMINAL (NRW_CONNECTION_NAME, yytext);
<SC_SQL>"CONSTRAINT_CATALOG" 		RETURN_LITERAL_TERMINAL (NRW_CONSTRAINT_CATALOG, yytext);
<SC_SQL>"CONSTRAINT_NAME"		RETURN_LITERAL_TERMINAL (NRW_CONSTRAINT_NAME, yytext);
<SC_SQL>"CONSTRAINT_SCHEMA" 		RETURN_LITERAL_TERMINAL (NRW_CONSTRAINT_SCHEMA, yytext);
<SC_SQL>"CURSOR_NAME"			RETURN_LITERAL_TERMINAL (NRW_CURSOR_NAME, yytext);
<SC_SQL>"DATA"				RETURN_LITERAL_TERMINAL (NRW_DATA, yytext);
<SC_SQL>"DATETIME_INTERVAL_CODE"	RETURN_LITERAL_TERMINAL (NRW_DATETIME_INTERVAL_CODE, yytext);
<SC_SQL>"DATETIME_INTERVAL_PRECISION"	RETURN_LITERAL_TERMINAL (NRW_DATETIME_INTERVAL_PRECISION, yytext);
<SC_SQL>"DYNAMIC_FUNCTION"		RETURN_LITERAL_TERMINAL (NRW_DYNAMIC_FUNCTION, yytext);
<SC_SQL>"LENGTH"			RETURN_LITERAL_TERMINAL (NRW_LENGTH, yytext);
<SC_SQL>"MESSAGE_LENGTH" 		RETURN_LITERAL_TERMINAL (NRW_MESSAGE_LENGTH, yytext);
<SC_SQL>"MESSAGE_OCTET_LENGTH" 		RETURN_LITERAL_TERMINAL (NRW_MESSAGE_OCTET_LENGTH, yytext);
<SC_SQL>"MESSAGE_TEXT"			RETURN_LITERAL_TERMINAL (NRW_MESSAGE_TEXT, yytext);
<SC_SQL>"MORE"				RETURN_LITERAL_TERMINAL (NRW_MORE, yytext);
<SC_SQL>"NAME"				RETURN_LITERAL_TERMINAL (NRW_NAME, yytext);
<SC_SQL>"NULLABLE" 			RETURN_LITERAL_TERMINAL (NRW_NULLABLE, yytext);
<SC_SQL>"NUMBER"			RETURN_LITERAL_TERMINAL (NRW_NUMBER, yytext);
<SC_SQL>"REPEATABLE" 			RETURN_LITERAL_TERMINAL (NRW_REPEATABLE, yytext);
<SC_SQL>"RETURNED_LENGTH" 		RETURN_LITERAL_TERMINAL (NRW_RETURNED_LENGTH, yytext);
<SC_SQL>"RETURNED_OCTET_LENGTH" 	RETURN_LITERAL_TERMINAL (NRW_RETURNED_OCTET_LENGTH, yytext);
<SC_SQL>"RETURNED_SQLSTATE"		RETURN_LITERAL_TERMINAL (NRW_RETURNED_SQLSTATE, yytext);
<SC_SQL>"ROW_COUNT"			RETURN_LITERAL_TERMINAL (NRW_ROW_COUNT, yytext);
<SC_SQL>"SCALE"				RETURN_LITERAL_TERMINAL (NRW_SCALE, yytext);
<SC_SQL>"SCHEMA_NAME" 			RETURN_LITERAL_TERMINAL (NRW_SCHEMA_NAME, yytext);
<SC_SQL>"SERIALIZABLE" 			RETURN_LITERAL_TERMINAL (NRW_SERIALIZABLE, yytext);
<SC_SQL>"SERVER_NAME"			RETURN_LITERAL_TERMINAL (NRW_SERVER_NAME, yytext);
<SC_SQL>"SUBCLASS_ORIGIN"		RETURN_LITERAL_TERMINAL (NRW_SUBCLASS_ORIGIN, yytext);
<SC_SQL>"TABLE_NAME"			RETURN_LITERAL_TERMINAL (NRW_TABLE_NAME, yytext);
<SC_SQL>"TYPE"				RETURN_LITERAL_TERMINAL (NRW_TYPE, yytext);
<SC_SQL>"UNCOMMITTED" 			RETURN_LITERAL_TERMINAL (NRW_UNCOMMITTED, yytext);
<SC_SQL>"UNNAMED"			RETURN_LITERAL_TERMINAL (NRW_UNNAMED, yytext);

<SC_SQL>{fourth_level_asterisk}			return ptApplication->identifyToken (yytext, 3);
<SC_SQL>{third_level_asterisk}			return ptApplication->identifyToken (yytext, 2);
<SC_SQL>{second_level_asterisk}			return ptApplication->identifyToken (yytext, 1);
<SC_SQL>{fourth_level_identifier}		return ptApplication->identifyToken (yytext, 3);
<SC_SQL>{third_level_identifier}		return ptApplication->identifyToken (yytext, 2);
<SC_SQL>{second_level_identifier}		return ptApplication->identifyToken (yytext, 1);
<SC_SQL>{identifier_body}			return ptApplication->identifyToken (yytext);
<SC_SQL>{delimited_identifier}			RETURN_STRING_LITERAL_TERMINAL (DELIMITED_IDENTIFIER, yytext);
<SC_SQL>{not_equals_operator}			RETURN_TERMINAL (NOT_EQUALS_OPERATOR);
<SC_SQL>{greater_than_or_equals_operator}	RETURN_TERMINAL (GREATER_THAN_OR_EQUALS_OPERATOR);
<SC_SQL>{less_than_or_equals_operator}		RETURN_TERMINAL (LESS_THAN_OR_EQUALS_OPERATOR);
<SC_SQL>{concatenation_operator}		RETURN_TERMINAL (CONCATENATION_OPERATOR);
<SC_SQL>{separator}				;


	/*
	*  Subsection 5.3. <literal>. 
	*/

<SC_SQL>{national_character_string_literal}	RETURN_STRING_LITERAL_TERMINAL (NATIONAL_CHARACTER_STRING_LITERAL, yytext);
<SC_SQL>{bit_string_literal}			RETURN_STRING_LITERAL_TERMINAL (BIT_STRING_LITERAL, yytext);
<SC_SQL>{hex_string_literal}			RETURN_STRING_LITERAL_TERMINAL (HEX_STRING_LITERAL, yytext);
<SC_SQL>{character_string_literal}		RETURN_STRING_LITERAL_TERMINAL (CHARACTER_STRING_LITERAL, yytext);
<SC_SQL>{approximate_numeric_literal}		RETURN_LITERAL_TERMINAL (APPROXIMATE_NUMERIC_LITERAL, yytext);
<SC_SQL>{unsigned_integer}			RETURN_LITERAL_TERMINAL (UNSIGNED_INTEGER, yytext);
<SC_SQL>{date_string}				RETURN_LITERAL_TERMINAL (DATE_STRING, yytext);
<SC_SQL>{time_string}				RETURN_LITERAL_TERMINAL (TIME_STRING, yytext);
<SC_SQL>{timestamp_string}			RETURN_LITERAL_TERMINAL (TIMESTAMP_STRING, yytext);
<SC_SQL>{interval_string}			RETURN_LITERAL_TERMINAL (INTERVAL_STRING, yytext);
	/*
	*  REAL_EXACT_NUMERIC_LITERAL
	*
	*  Warning:  This terminal is not part of the specification.  It's used
        *            by <exact numeric literal> and by <C_numeric_literal>.
	*/
<SC_SQL>{real_exact_numeric_literal}		RETURN_LITERAL_TERMINAL (REAL_EXACT_NUMERIC_LITERAL, yytext);


	/*
	*  Next terminals aren't part of the specification.
	*  They are used for compiler purposes.
	*/
<SC_SQL>"}"					BEGIN (INITIAL); RETURN_TERMINAL (EMBEDDED_SQL_END);
<SC_SQL>.					RETURN_TERMINAL (LEXICAL_ERROR);


%%

/*
*  L E X   F U N C T I O N S
*/

/**
*  This function is called when the input finish, and if you want to continue
*  it musts return 1, or if you want to finish then it musts return 0.
*  @return One if you want to continue, and zero if you want to finish.
*/
int yywrap (void)
{

  return 1;

}  // yywrap()
